# -*- coding: utf-8 -*-
import numpy as np

def create(kind='hamming', fstop=0.1):
  '''
  Funkcija ustvari konvolucijsko jedro visokoprepustnega filtra.

  Parametri
  ---------
  kind: str
    Tip visokoprepustnega filtra, in sicer "ramp", "shepp-logan",
    "cosine", "hamming" ali "hann".
  fstop: float
    Delež frekvenčnega področja pod Nyquistovo mejo, ki ga
    odreže filter (od 0 do 1).

  Vrne
  ----
  kernel: vektor   
    Konvolucijsko jedro visokoprepustnega filtra.
  '''
  len = 64
  d = 1 - fstop
  order = max(64, 2**np.ceil(np.log2(2*len)))

  filt = 2.0*np.arange(0, order*0.5)/order
  w = 2.0*np.pi*np.arange(0, filt.size)/order  

  if kind == 'ramp':
    pass
  elif kind == 'shepp-logan':
    filt[1:] *= np.sin(w[1:]/(2.0*d))/(w[1:]/(2.0*d))
  elif kind == 'cosine':
    filt[1:] *= np.cos(w[1:]/(2.0*d))
  elif kind == 'hamming'  :
   filt[1:] *= 0.54 + 0.46*np.cos(w[1:]/d)
  elif kind == 'hann':
   filt[1:] *= (1.0 + np.cos(w[1:]/d))/2.0
  elif kind == 'none':
   filt.fill(1.0)
  else:
   raise ValueError(
     'Vrednost parametra "kind" je lahko '
     '"ramp", "shepp-logan", "hamming" ali "hann"!')

  filt[w > np.pi*d] = 0
  filt = np.hstack([filt, filt[-2::-1]])

  filter = (np.fft.ifft(filt))
  filterCutoff = 0.95
  filterAbs = np.abs(filter)/np.abs(filter).sum()
  filterSum = filterAbs[0]
  for i in range(int(np.floor(filterAbs.size*0.5))):
    filterSum += filterAbs[i + 1] + filterAbs[-1-i]
    if filterSum >= filterCutoff:
      break

  kernel = np.hstack([filter[-1 - i:], filter[:i + 1 + 1]])
  
  return np.real(kernel)

if __name__ == '__main__':
  k = create()