# -*- coding: utf-8 -*-
#%% Poglavje 11

import funkcije
import numpy as np
from matplotlib import pyplot as pp
from PIL import Image as im
import time

# naložimo 3D CT sliko
Ict = funkcije.imLoadRaw3d(
  './poglavje_11/ct_287x165x194_uint8.raw',
  287, 165, 194)
D, H, W = Ict.shape # velikost slike
dz, dy, dx = 4.0, 2.0, 2.0
# določimo vektorje, ki napenjajo koordinate slikovnih elementov
x = np.arange(-dx*(W - 1)*0.5, dx*(W - 1)*0.5 + 0.5*dx, dx)
y = np.arange(-dy*(H - 1)*0.5, dy*(H - 1)*0.5 + 0.5*dy, dy)
z = np.arange(-dz*(D - 1)*0.5, dz*(D - 1)*0.5 + 0.5*dz, dz)

# koti za katere bomo izračunali projekcije
fi = np.deg2rad(np.arange(0.0, 360.0, 5))

#%% Naloga 1
# V modulu funkcije ustvarimo funkcijo imParallelBeamProject3D.

# Naloga a)
IctP90 = funkcije.imParallelBeamProject3D(
  np.deg2rad(90), 2.0, 
  Ict, x, y, z, 800.0, 128, 128, 10.0, 'max')
pp.figure()
pp.imshow(np.squeeze(IctP90), cmap='gray')
pp.title('Pravokotna projekcija 90')
pp.show()

# Naloga b)
start = time.perf_counter()
IctP = funkcije.imParallelBeamProject3D(
  fi, 2.0, Ict, x, y, z, 800.0, 128, 128, 10.0, 'max')
dt = time.perf_counter() - start
print('{} pravokotnih projekcij izračunanih v '
      '{:.1f} s.'.format(fi.size, dt))

pp.figure()
pp.ion()

frame, title = None, None
for i in range(IctP.shape[2]):
  if frame is None:
    frame = pp.imshow(
      IctP[:,:,i], vmin=0, vmax=255, cmap='gray')
  else:
    frame.set_array(IctP[:,:,i])

  if title is None:
    title = pp.title('Fi: {:.0f}'.format(np.rad2deg(fi[i])))
  else:
    title.set_text('Fi: {:.0f}'.format(np.rad2deg(fi[i])))

  pp.draw()
  pp.pause(.1)

pp.ioff()

#%% Naloga 2
# V modulu funkcije ustvarimo funkcijo imFanBeamProject3D.

# Naloga a)
IctS90 = funkcije.imFanBeamProject3D(
  np.deg2rad(90), 2.0, 
  Ict, x, y, z, 800.0, 128, 128, 20.0, 'max')

pp.figure()
pp.imshow(np.squeeze(IctS90), cmap='gray')
pp.title('Pravokotna projekcija 90')
pp.show()

#%% Naloga b)
steps = [1, 2, 4, 8]
IctST = []
TSo = []
TSfb = []
pp.figure()
pp.suptitle('Vpliv vzorčnega koraka na izračun projekcij.')

for i in range(len(steps)):
  start = time.perf_counter()
  funkcije.imParallelBeamProject3D(
    np.deg2rad(90), steps[i],
    Ict, x, y, z, 800.0, 128, 128, 10.0, 'max')
  TSo.append(time.perf_counter() - start)
  print('Pravokotna projekcija s korakom vzorčenja {} mm '
        'izračunana v {:.0f} ms'.format(steps[i], TSo[-1]*1000))
  
  start = time.perf_counter()
  tmp = funkcije.imFanBeamProject3D(
    np.deg2rad(90), steps[i], 
    Ict, x, y, z, 800.0, 128, 128, 20.0, 'max')
  IctST.append(np.squeeze(tmp))
  TSfb.append(time.perf_counter() - start)
  print('Stožčasta rojekcija s korakom vzorčenja {} mm '
        'izračunana v {:.0f} ms'.format(steps[i], TSfb[-1]*1000))
  pp.subplot(1, 4, i + 1)
  pp.imshow(IctST[-1], cmap='gray')
  pp.title('{} mm'.format(steps[i]))

pp.show()

# Naloga c)
start = time.perf_counter()
IctS = funkcije.imFanBeamProject3D(
  fi, 2.0, Ict, x, y, z, 800.0, 128, 128, 20.0, 'max')
dt = time.perf_counter() - start
print('{} stožčastih projekcij izračunanih '
      'v {:.1f} s.'.format(fi.size, dt))

pp.figure()
pp.ion()

frame, title = None, None
for i in range(IctS.shape[2]):
  if frame is None:
    frame = pp.imshow(
      IctS[:,:,i], vmin=0, vmax=255, cmap='gray')
  else:
    frame.set_array(IctS[:,:,i])

  if title is None:
    title = pp.title('Fi: {:.0f}'.format(np.rad2deg(fi[i])))
  else:
    title.set_text('Fi: {:.0f}'.format(np.rad2deg(fi[i])))

  pp.draw()
  pp.pause(.1)

pp.ioff()
