# -*- coding: utf-8 -*-
#%% Poglavje 12

import funkcije
import numpy as np
from matplotlib import pyplot as pp
from PIL import Image as im
import time
import hpfilter
import os.path

# naloži testno slike in pripravi koordinatno mrežo
It = np.asarray(im.open('./poglavje_12/bproj_test.png'))
xt = np.arange(It.shape[1], dtype=np.float)
yt = np.arange(It.shape[0], dtype=np.float)
xt -= xt.mean()
yt -= yt.mean()

# naloži rezino CT slike in pripravi koordinatno mrežo
Ict = np.asarray(im.open('./poglavje_12/bproj_ct.png'))
xct = 2.0*np.arange(Ict.shape[1], dtype=np.float) 
yct = 2.0*np.arange(Ict.shape[0], dtype=np.float)
xct -= xct.mean()
yct -= yct.mean()

fi = np.deg2rad(np.arange(0, 360))

# geometrija pravokotne projekcije
dP, nP, psP = 1000, 512, 1
# geometrija Stožčaste projekcije
dS, nS, psS = 1000, 512, 2

# ponovno računanje sinogramov
recompute = True

#%% Naloga 1
# V modulu funkcije ustvarimo funkcijo imParallelBeamBackproject2D.

# izračunaj sinogram, če je to potrebno
if os.path.isfile('./poglavje_12/rezultati/ItSP.npy') and not recompute:
  ItSP = np.load('./poglavje_12/rezultati/ItSP.npy')
else:
  ItSP = funkcije.imParallelBeamProject2d(fi, 1, It,
    xt, yt, dP, nP, psP, 'mean')
  np.save('./poglavje_12/rezultati/ItSP.npy', ItSP)
    
start = time.perf_counter()
inds = np.arange(360)
ItBpP = funkcije.imParallelBeamBackproject2D(
  fi[inds], ItSP[inds, :], xt, yt, dP, psP)
T = time.perf_counter() - start

pp.figure()

pp.subplot(1, 3, 1)
pp.suptitle('Testna slika - pravokotna projekcija')
pp.imshow(It, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(ItSP, cmap='gray')
pp.title('Sinogram')

pp.subplot(1, 3, 3)
pp.imshow(ItBpP, cmap='gray')
pp.title('Povratna projekcija')

pp.show()

# Naloga b)
# izračunaj sinogram, če je to potrebno
if os.path.isfile('./poglavje_12/rezultati/IctSP.npy') and not recompute:
  IctSP = np.load('./poglavje_12/rezultati/IctSP.npy')
else:
  IctSP = funkcije.imParallelBeamProject2d(
    fi, 1, Ict, xct, yct, dP, nP, psP, 'mean')
  np.save('./poglavje_12/rezultati/IctSP.npy', IctSP)

start = time.perf_counter()
inds = np.arange(360)
IctBpP = funkcije.imParallelBeamBackproject2D(fi[inds], 
  IctSP[inds, :], xct, yct, dP, psP)
T = time.perf_counter() - start

pp.figure()

pp.subplot(1, 3, 1)
pp.suptitle('Rezina CT slike - pravokotna projekcija')
pp.imshow(Ict, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(IctSP, cmap='gray')
pp.title('Sinogram')

pp.subplot(1, 3, 3)
pp.imshow(IctBpP, cmap='gray')
pp.title('Povratna projekcija')

pp.show()

#%% Naloga 2
# V modulu funkcije ustvarimo funkcijo imFanBeamBackproject2D.

# Naloga a)
# izračunaj sinogram, če je to potrebno
if os.path.isfile('./poglavje_12/rezultati/ItSS.npy') and not recompute:
  ItSS = np.load('./poglavje_12/rezultati/ItSS.npy')
else:
  ItSS = funkcije.imFanBeamProject2d(fi, 1, It,
    xt, yt, dS, nS, psS, 'mean')
  np.save('./poglavje_12/rezultati/ItSS.npy', ItSS)
    
start = time.perf_counter()
inds = np.arange(360)
ItBpS = funkcije.imFanBeamBackproject2D(
  fi[inds], ItSS[inds, :], xt, yt, dS, psS)
T = time.perf_counter() - start

pp.figure()

pp.subplot(1, 3, 1)
pp.suptitle('Testna slika - stožčasta projekcija')
pp.imshow(It, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(ItSS, cmap='gray')
pp.title('Sinogram')

pp.subplot(1, 3, 3)
pp.imshow(ItBpS, cmap='gray')
pp.title('Povratna projekcija')

pp.show()

# Naloga b)
# izračunaj sinogram, če je to potrebno
if os.path.isfile('./poglavje_12/rezultati/IctSS.npy') and not recompute:
  IctSS = np.load('./poglavje_12/rezultati/IctSS.npy')
else:
  IctSS = funkcije.imFanBeamProject2d(fi, 2, Ict,
    xct, yct, dS, nS, psS, 'mean')
  np.save('./poglavje_12/rezultati/IctSS.npy', IctSS)

start = time.perf_counter()
inds = np.arange(360)
IctBpS = funkcije.imFanBeamBackproject2D(fi[inds], 
  IctSS[inds, :], xct, yct, dS, psS)
T = time.perf_counter() - start

pp.figure()

pp.subplot(1, 3, 1)
pp.suptitle('Rezina CT slike - stožčasta projekcija')
pp.imshow(Ict, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(IctSS, cmap='gray')
pp.title('Sinogram')

pp.subplot(1, 3, 3)
pp.imshow(IctBpS, cmap='gray')
pp.title('Povratna projekcija')

pp.show()

#%% Naloga 3

# Naloga a) in b)
filt = hpfilter.create('hamming')

# filtriramo sinograme
ItSPF = funkcije.filterSinogram(ItSP, filt)
ItSSF = funkcije.filterSinogram(ItSS, filt)

IctSPF = funkcije.filterSinogram(IctSP, filt)
IctSSF = funkcije.filterSinogram(IctSS, filt)

inds = np.arange(360)
# rekonstrukcija s filtriranimi sinogrami
ItFBpP = funkcije.imParallelBeamBackproject2D(
  fi[inds], ItSPF[inds, :], xt, yt, dP, psP)
ItFBpS = funkcije.imFanBeamBackproject2D(
  fi[inds], ItSSF[inds, :], xt, yt, dS, psS)

IctFBpP = funkcije.imParallelBeamBackproject2D(
  fi[inds], IctSPF[inds, :], xct, yct, dP, psP)
IctFBpS = funkcije.imFanBeamBackproject2D(
  fi[inds], IctSSF[inds, :], xct, yct, dS, psS)

# testna slika
pp.figure()
pp.suptitle(
  'Primerjava povratnih projekcij na podlagi '
  'filtriranih in nefiltriranih sinogramov - '
  'pravokotna projekcija')

pp.subplot(1, 3, 1)
pp.imshow(It, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(ItBpP, cmap='gray')
pp.title('Nefiltrirana VP')

pp.subplot(1, 3,3)
pp.imshow(ItFBpP, cmap='gray')
pp.title('Filtrirana VP')

pp.figure()
pp.suptitle(
  'Primerjava povratnih projekcij na podlagi '
  'filtriranih in nefiltriranih sinogramov - '
  'stožčasta projekcija')

pp.subplot(1, 3, 1)
pp.imshow(It, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(ItBpS, cmap='gray')
pp.title('Nefiltrirana VP')

pp.subplot(1, 3, 3)
pp.imshow(ItFBpS, cmap='gray')
pp.title('Filtrirana VP')

# CT rezina
pp.figure()
pp.suptitle(
  'Primerjava povratnih projekcij na podlagi '
  'filtriranih in nefiltriranih sinogramov - '
  'pravokotna projekcija')

pp.subplot(1, 3, 1)
pp.imshow(Ict, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(IctBpP, cmap='gray')
pp.title('Nefiltrirana VP')

pp.subplot(1, 3, 3)
pp.imshow(IctFBpP, cmap='gray')
pp.title('Filtrirana VP')

pp.figure()
pp.suptitle(
  'Primerjava povratnih projekcij na podlagi '
  'filtriranih in nefiltriranih sinogramov - '
  'stožčasta projekcija')

pp.subplot(1, 3, 1)
pp.imshow(Ict, cmap='gray')
pp.title('Izvirna')

pp.subplot(1, 3, 2)
pp.imshow(IctBpS, cmap='gray')
pp.title('Nefiltrirana VP')

pp.subplot(1, 3, 3)
pp.imshow(IctFBpS, cmap='gray')
pp.title('Filtrirana VP')

pp.show()

#%% Naloga 4
r = [1, 2, 4, 8]
IctFbpFi = []
IctPbpFi = []

pp.figure()
pp.suptitle(
  'Vpliv števila projekcij na '
  'kakovost povratne projekcije.')

pp.subplot(1, len(r) + 1, 1)
pp.imshow(Ict, cmap='gray')
pp.title('Izvirna')

TSo = []
TSfb = []
for i in range(len(r)):
  inds = np.arange(0, 360, r[i])

  tstart = time.perf_counter()
  tmp = funkcije.imParallelBeamBackproject2D(
    fi[inds], IctSP[inds, :], xct, yct, dP, psP)
  IctPbpFi.append(tmp) 
  TSo.append(time.perf_counter() - tstart)
  print('Pravokotna povratna projekcija s korakom vzorčenja {}° '
        'izračunana v {:.0f} ms'.format(r[i], TSo[-1]*1000))

  tstart = time.perf_counter()
  tmp = funkcije.imFanBeamBackproject2D(
    fi[inds], IctSSF[inds, :], xct, yct, dS, psS)
  IctFbpFi.append(tmp) 
  TSfb.append(time.perf_counter() - tstart)
  print('Stožčasta povratna projekcija s korakom vzorčenja {}° '
        'izračunana v {:.0f} ms'.format(r[i], TSfb[-1]*1000))

  pp.subplot(1, len(r) + 1, i + 2)
  pp.imshow(IctFbpFi[-1], cmap='gray')
  pp.title('Korak {} stopinj'.format(r[i]))

pp.show()
