
# -*- coding: utf-8 -*-
#%% Poglavje 2

import numpy as np
from matplotlib import pyplot as pp
import funkcije

im1 = funkcije.imLoadRaw2d(
  './poglavje_2/mr_256x256_uint8.raw', 256, 256)
im2 = funkcije.imLoadRaw2d(
  './poglavje_2/mr_128x256_uint8.raw', 128, 256)
im3 = funkcije.imLoadRaw3d(
  './poglavje_2/mr_217x181x181_uint8.raw', 217, 181, 181)
im4 = funkcije.imLoadRaw2d(
  './poglavje_2/decimacija_256x256_uint8.raw', 256, 256)

#%% Naloga 1

# Funkcijo intep1d ustvarimo v modulu funkcije.

# Glavna prednost interpolacije z metodo najbližjega soseda leži v računski
# nezahtevnosti postopka, slabost pa v nizki kakovosti interpolirane slike.
# Linearna interpolacija je približno 4-krat zahtevnejša od interpolacije
# z metodo najbližjega soseda. Kakovost interpoliranih slik je bistveno boljša,
# saj postopek upošteva uteženo vsoto sivinskih vrednost 4-ih sosednjih
# slikovnih elementov.

# preizkusimo funkcijo interp1d

xp = np.linspace(0, np.pi, 10)
yp = np.sin(xp)
x = np.linspace(0, np.pi, 30)

pp. figure('Preizkus interp1d')

pp.subplot(1, 2, 1)
y = funkcije.interp1d(x, xp, yp, 'nearest')
pp.title('Interpolacija z najbližjim sosedom')
pp.plot(xp, yp, '-k*', label='podatki')
pp.plot(x, y, '-r.', label='interpolacija')
pp.legend()

pp.subplot(1, 2, 2)
y = funkcije.interp1d(x, xp, yp, 'linear')
pp.title('Linearna interpolacija')
pp.plot(xp, yp, '-k*', label='podatki')
pp.plot(x, y, '-r.', label='interpolacija')
pp.legend()

pp.show()
#%% Naloga 2

# Funkcijo intep2d ustvarimo v modulu funkcije.

# Naloga a)
x = y = np.linspace(0, 255, 512)
xp = yp = np.arange(256)
Yi, Xi = np.meshgrid(y, x, indexing='ij')
im1i_linear = funkcije.interp2d(Xi, Yi, xp, yp, im1, 'linear')
im1i_nearest = funkcije.interp2d(Xi, Yi, xp, yp, im1, 'nearest')

pp.figure('Odgovor 2 a)')

pp.subplot(1, 2, 1)
pp.imshow(im1i_linear, cmap='gray')
pp.title('Linearna')

pp.subplot(1, 2, 2)
pp.imshow(im1i_nearest, cmap='gray')
pp.title('Najbližji sosed')

pp.show()
# Naloga b)
xp = np.arange(0, 256, 2)
yp = np.arange(256) 
x = np.arange(0, 256)
y = yp
Yi, Xi = np.meshgrid(y, x,  indexing='ij')
im2i_linear = funkcije.interp2d(Xi, Yi, xp, yp, im2, 'linear')
im2i_nearest = funkcije.interp2d(Xi, Yi, xp, yp, im2, 'nearest')

pp.figure('Odgovor 2 b)')

pp.subplot(1, 2, 1)
pp.imshow(im2i_linear, cmap='gray')
pp.title('Linearna')

pp.subplot(1, 2, 2)
pp.imshow(im2i_nearest, cmap='gray')
pp.title('Najbližji sosed')

pp.show()

#%% Naloga c)
xi = np.arange(128 - 16*2, 128 + 16*2, 0.25)
yi = np.arange(128 - 32, 128 + 32, 0.25)
Yi, Xi = np.meshgrid(yi, xi, indexing='ij')

im2i_box_nearest = funkcije.interp2d(Xi, Yi, xp, yp, im2, 'nearest')
im2i_box_linear = funkcije.interp2d(Xi, Yi, xp, yp, im2, 'linear')

from matplotlib.patches import Rectangle
region = Rectangle((64 - 16, 128 - 32), 32, 64,
                   edgecolor='b', facecolor='none')

pp.figure('Odgovor 2 c)')
pp.suptitle('Interpolacija osrednjega dela slike')

pp.subplot(1, 3, 1)
pp.imshow(im2, cmap='gray', vmin=0, vmax=255)
pp.gca().add_patch(region)
pp.title('Izhodiščna slika')

pp.subplot(1, 3, 2)
pp.title('Najbližji sosed')
pp.imshow(im2i_box_nearest, cmap='gray', vmin=0, vmax=255)

pp.subplot(1, 3, 3)
pp.title('Linearna')
pp.imshow(im2i_box_linear, cmap='gray', vmin=0, vmax=255)

pp.show()
#%% Naloga 3

# Funkcijo intep3d ustvarimo v modulu funkcije.

xp = np.arange(217)
yp = np.arange(181)
zp = np.arange(181)
y = z = np.linspace(0, 180, 181*2)
Zi, Yi, Xi = np.meshgrid(z, y, 110.3, indexing='ij')
im3i_nearest = funkcije.interp3d(Xi, Yi, Zi, xp, yp, zp, im3, 'nearest')
im3i_linear = funkcije.interp3d(Xi, Yi, Zi, xp, yp, zp, im3, 'linear')

pp.figure('Odgovor 3')

pp.subplot(1, 2, 1)
pp.imshow(im3i_linear.squeeze(), cmap='gray')
pp.title('Linearna pri z=110.3')

pp.subplot(1, 2, 2)
pp.imshow(im3i_nearest.squeeze(), cmap='gray')
pp.title('Najbližji sosed pri z=110.3')

pp.show()

#%% Naloga 4

# Funkcijo conv2d ustvarimo v modulu funkcije.

#%% Naloga 5

# Funkcijo imDecimate2d ustvarimo v modulu funkcije.

K3 = np.array([[1.0/16, 1/8, 1/16],
               [1/8,    1/4, 1/8],
               [1.0/16, 1/8, 1/16]])
K5 = np.array([[1.0/400, 1/80, 1/50, 1/80, 1/400],
               [1/80,    1/16, 1/10, 1/16, 1/80],
               [1/50,    1/10, 4/25, 1/10, 1/50],
               [1/80,    1/16, 1/10, 1/16, 1/80],
               [1.0/400, 1/80, 1/50, 1/80, 1/400]])

im4d_k3 = funkcije.imDecimate2d(im4, K3 ,2)
im4d = im4[::4,::4]
im1d_k3 = funkcije.imDecimate2d(im1, K3 ,2)
im1d = im1[::4,::4]

pp.figure('Odgovor 5')

pp.subplot(2, 3, 1)
pp.imshow(im1, cmap='gray')
pp.title('Izvirna slika')

pp.subplot(2, 3, 2)
pp.imshow(im1d_k3, cmap='gray')
pp.title('Decimimacija s filtranjem')

pp.subplot(2, 3, 3)
pp.imshow(im1d, cmap='gray')
pp.title('Decimirana brez filtriranja')

pp.subplot(2, 3, 4)
pp.imshow(im4, cmap='gray')
pp.title('Izvirna slika')

pp.subplot(2, 3, 5)
pp.imshow(im4d_k3, cmap='gray')
pp.title('Decimimacija s filtranjem')

pp.subplot(2, 3, 6)
pp.imshow(im4d, cmap='gray')
pp.title('Decimirana brez filtranja')

pp.show()
