# -*- coding: utf-8 -*-
#%% Poglavje 3

from matplotlib import pyplot as pp
import numpy as np
from PIL import Image as im
import funkcije

# Naložimo sliki kalibrov.
CC = np.asarray(im.open('./poglavje_3/ColorChecker.jpg'))
ISO = np.asarray(im.open('./poglavje_3/ISO-12233.jpg'))

# Pretvorimo sliki v sivinski.
CCg, ISOg = funkcije.srgb2gs(CC), funkcije.srgb2gs(ISO)

# Število podslik v slikah A in B.
Nx, Ny = 6, 4
N = Nx*Ny
kxy = 0.5	# delež izrezanega področja

# Velikost slik A in B.
H, W = CC.shape[0], CC.shape[1]

# Preslikava iz sRGB v XYZ pri beli referenci D65.
RGB2XYZ = np.array([[0.4124564, 0.3575761, 0.1804375],
                    [0.2126729, 0.7151522, 0.0721750],
                    [0.0193339, 0.1191920, 0.9503041]])

# Funkcija za določanje področij na sliki.
def extractSubImages(img, t=None, show=False, k=0.5):
  '''
  Funkcija izlušči 24 slik področij, ki pripadajo posameznim barvam in 
  sivinskim odtekom kalibra "ColorChecker". V ta namen je potrebno označiti
  središče levega zgornjega (rjavega) področja in desnega spodnjega (črnega)
  področja. Algoritem predpostavlja, da so robovi kalibra dobro poravnani z
  robovi slike.

  Parametri
  ---------
  img: numpy.ndarray
    Slika kalibra "ColorChecker".
  t:  list
    Seznam s koordinatami točk levega zgornjega (rjavega) in desnega
    spodnjega področja (črnega), ki ga vrne funkcija ginput.
  show: bool, neobvezno
    Če je vrednost True, potem v grafičnem oknu prikaže izluščene slike
    področij.
  k: float
    Delež izrezanega področja v smeri x in y koordinatne osi.
  Vrne
  ----
  subimg: list
    Seznam 24-ih slik področij kalibra "ColorChecker" v vrstnem redu, ki je
    označen v navodilih laboratorijske vaje.
  t: list
    Seznam označenih točk, ki jih pri naslednjem klicu lahko podamo
    funkciji, v izogib ročnemu označevanju točk.

  Primeri uporabe:
  ---------------
  >>> import funkcije
  >>> from PIL import Image as im
  >>> import numpy as np
  >>>
  >>> cc = np.array(im.open("./vaja2/ColorChecker.jpg"))
  >>> reg, t = extractSubImages(cc, show=True)
  >>>
  '''
  img = np.asarray(img)  
  Ny = 4
  Nx = 6
  if t is None:
    pp.figure()
    pp.imshow(img)
    pp.title('Označi središče levega zgornjega (rjavega) in, '\
        'nato desnega spodnjega (črnega) področja.')
    t1, t2 = pp.ginput(2)
  else:
    t1, t2 = t

  if show:
    pp.figure()
    pp.title("Izluščene slike področij")

  h = (t2[1] - t1[1])/float(Ny - 1) # višina področja
  w = (t2[0] - t1[0])/float(Nx - 1) # širina področja
  
  x0 = t1[0] # x koordinata središča področja 1
  y0 = t1[1] # y koordinata središča področja 1
  
  h2 = int(round(k*h/2))
  w2 = int(round(k*w/2))
  podslike = []
  for j in range(Ny): # naslov področja v navpični (y) smeri
    for i in range(Nx): # naslov področja v vodoravni (x) smeri
      xcenter = int(round(x0 + w*i)) # odmik središča v vodoravne smeri
      ycenter = int(round(y0 + h*j)) # odmik središča v navpični smeri
      podslike.append( # sliko področja pripnemo na koncu seznama
        img[ycenter - h2 : ycenter + h2,
            xcenter - w2 : xcenter + w2])

      if show:
        pp.subplot(Ny, Nx, i + j*Nx + 1)
        pp.imshow(podslike[-1])
        pp.axis('off')

  if show is None:
    pp.suptitle('Izbrana področja.')

  return podslike, (t1,t2,)

# Pomožna funkcija za pretvorbo iz XYZ v Lab barvni prostor.
def funXYZ2Lab(t):
  out = np.zeros_like(t)
  ind = t > (6.0/29.0)**3
  out[ind] = t[ind]**(1.0/3.0)
  nind = np.logical_not(ind)
  out[nind] = (1.0/3.0*(29.0/6.0)**2)*t[nind] + 4.0/29.0

  return out

# Funkcija za pretvorbo iz XYZ v Lab barvni prostor.
def Xyz2Lab(X, Y, Z, Xw, Yw, Zw):
  tx = X/Xw
  ty = Y/Yw
  tz = Z/Zw

  L = 116.0*funXYZ2Lab(tx) - 16.0
  a = 500*(funXYZ2Lab(tx) - funXYZ2Lab(ty))
  b = 200*(funXYZ2Lab(ty) - funXYZ2Lab(tz))

  return L, a, b

t1iso = t2iso = None
rectCC = rectISO = None

#%% Naloga 1
sCC, rectCC = extractSubImages(CC, t=rectCC, show=True, k=kxy)

#%% Naloga 2
sCCg = []
sCCmean = np.zeros(N)
sCCstd = np.zeros(N)
for i in range(len(sCC)):
  cc = sCC[i]
  sCCg.append(funkcije.srgb2gs(cc))
  sCCmean[i] = sCCg[-1].mean()
  sCCstd[i] = sCCg[-1].std()

#%% Naloga 3
pp.figure()

pp.subplot(3, 1, 1)
pp.imshow(ISO, cmap='gray')
pp.title('Označi krajišča prereza - levo zgoraj nato desno spodaj')
if t1iso is None or t2iso is None:
  t1iso, t2iso = pp.ginput(n=2, timeout=120)
  t1iso = [int(t1iso[0]), int(t1iso[1])]
  t2iso = [int(t2iso[0]), int(t2iso[1])]
pp.plot([t1iso[0], t2iso[0]], [t1iso[1], t2iso[1]], '-r')

pp.subplot(3, 1, 2)
mtfprerez = ISOg[t1iso[1]:max(t1iso[1] + 1, t2iso[1]),
                 t1iso[0]:t2iso[0]].mean(0)
isolp = np.linspace(1, 10, mtfprerez.size)
pp.plot(isolp, mtfprerez)
pp.title('Izvirni prerez')

pp.subplot(3, 1, 3)
pp.title('Normalizirani prerez')
l, h = mtfprerez.min(), mtfprerez.max()
pp.plot(isolp, (mtfprerez - l)/(h - l) - 0.5, '-r')
pp.plot([1, 10], [0.05, 0.05], '-g')
pp.plot([1, 10], [-0.05, -0.05], '-g')

pp.show()
# Vrednost  modulacijske prenosne funkcije pade na 10 % začetne vrednosti pri
#   približno 750 progah na višino, torej pri širini proge 200/750 mm ~ 266 um.

#%% Naloga 4
pp.figure()
pp.plot(np.arange(19, 25), sCCmean[18:24], '-or')
pp.xlabel('Področje slike ColorChecker')
pp.ylabel('Povprečna sivina')
pp.show()
print(
  'Dinamično območje znaša {:.1f} bitov.'.format(
    np.log2(sCCmean[18] - sCCmean[23])))

#%% Naloga 5
# Funkcijo imHistogram ustvarimo v modulu funkcije.

hA, eA = funkcije.imHistogram(sCCg[18], span=[0,255], density=True)
hB, eB = funkcije.imHistogram(sCCg[23], span=[0,255], density=True)

pp.figure()

pp.subplot(1, 2, 1)
pp.bar(eA, hA, width=eA[1]-eA[0])
pp.plot(eA, 1.0/(sCCstd[18]*np.sqrt(2.0*np.pi))* \
    np.exp(-(eA - sCCmean[18])**2/2.0/sCCstd[18]**2), '-r')
pp.title('Področje 19')

pp.subplot(1, 2, 2)
pp.bar(eB, hB, width=eB[1]-eB[0])
pp.plot(eB, 1.0/(sCCstd[23]*np.sqrt(2.0*np.pi))* \
  np.exp(-(eB - sCCmean[23])**2/2.0/sCCstd[23]**2), '-r')
pp.title('Področje 24')

pp.show()

#%% Naloga 6
SNRda = (sCCmean[18] - sCCmean[23])/ \
       (sCCstd[18]**2 + sCCstd[23]**2)**0.5

SNRdb = (sCCmean[22] - sCCmean[23])/ \
       (sCCstd[22]**2 + sCCstd[23]**2)**0.5

print('SNRd za področji 19-24: {:.1f}'.format(SNRda))
print('SNRd za področji 23-24: {:.1f}'.format(SNRdb))

#%% Naloga 7
cR, cG, cB = CC[:,:,0], CC[:,:,1], CC[:,:,2]
rgb = np.vstack(
  [cR.flatten(), cG.flatten(), cB.flatten()])

xyz = np.dot(RGB2XYZ, rgb/255.0)
Xw, Yw, Zw = np.dot(RGB2XYZ, np.ones([3, 1]))
X = xyz[0].reshape(H, W)
Y = xyz[1].reshape(H, W)
Z = xyz[2].reshape(H, W)
x, y, z = X/(X + Y + Z), Y/(X + Y + Z), Z/(X + Y + Z)
XYZ = np.dstack((X, Y, Z))

# Naloga a)
pp.figure()

pp.subplot(1, 3, 1)
pp.imshow(Y, cmap='gray')
pp.title('Y')
pp.axis('off')

pp.subplot(1, 3, 2)
pp.imshow(x, cmap='gray')
pp.title('x')
pp.axis('off')

pp.subplot(1, 3, 3)
pp.imshow(y, cmap='gray')
pp.title('y')
pp.axis('off')

pp.show()

# Naloga b)
L, a, b = Xyz2Lab(X, Y, Z, Xw, Yw, Zw)
Lab = np.dstack((L, a, b))

pp.figure()

pp.subplot(1, 3, 1)
pp.imshow(L, cmap='gray')
pp.title('L')
pp.axis('off')

pp.subplot(1, 3, 2)
pp.imshow(a, cmap='gray')
pp.title('a')
pp.axis('off')

pp.subplot(1, 3, 3)
pp.imshow(b, cmap='gray')
pp.title('b')
pp.axis('off')

pp.show()

#%% Naloga 8
imga = im.open('./poglavje_1/mrBrainSlice.png')

quality = [25, 50, 75, 100]

pp.figure()

jpegs = []
ssims = []
tmpfile = './poglavje_3/rezultati/tmp.jpeg'
for index, q in enumerate(quality):
  imga.save(tmpfile, quality=q)
  jpegs.append(np.array(im.open(tmpfile)))

  pp.subplot(1, len(quality) + 1, 1 + index)
  ssims.append(funkcije.ssim(imga, jpegs[-1]))
  pp.imshow(ssims[-1], cmap='gray', vmin=0, vmax=1)
  pp.title('SSIM pri kakovosti {:d} %'.format(q))

pp.subplot(1, len(quality) + 1, len(quality) + 1)
pp.colorbar()
pp.axis('off')
pp.show()
