 #-*- coding: utf-8 -*-
#%% Poglavje 4

import funkcije
import numpy as np
from matplotlib import pyplot as pp
from mpl_toolkits.mplot3d import Axes3D

Knp = 1.0/9.0*np.array([[1, 1, 1],
                        [1, 1, 1],
                        [1, 1, 1]])
Kup = 1.0/16.0*np.array([[1, 2, 1],
                         [2, 4, 2],
                         [1, 2, 1]])
Kg = np.array([[0.01, 0.08, 0.01],
               [0.08, 0.64, 0.08],
               [0.01, 0.08, 0.01]])

I1 = funkcije.imLoadRaw2d(
  './poglavje_4/ct_175x175_uint8.raw', 175, 175)
I2 = funkcije.imLoadRaw2d(
  './poglavje_4/ct_sp_175x175_uint8.raw', 175, 175)
I3 = funkcije.imLoadRaw3d(
  './poglavje_4/mr_217x181x181_uint8.raw', 217, 181, 181)

#%% Naloga 1
# Izvedemo filtriranje s predlaganimi jedri.
I1np = np.round(funkcije.conv2d(I1, Knp)).astype(np.uint8)
I1up = np.round(funkcije.conv2d(I1, Kup)).astype(np.uint8)
I1g = np.round(funkcije.conv2d(I1, Kg)).astype(np.uint8)
# Prikažemo izvirno in vse tri zglajene slike.

pp.figure()
pp.suptitle('Primerjava treh nizkoprepustnih sit.')

pp.subplot(1, 4, 1)
pp.title('Izvirna slika')
pp.imshow(I1, cmap='gray')
pp.axis('off')

pp.subplot(1, 4, 2)
pp.imshow(I1np, cmap='gray')
pp.title('Navadno povprecje')
pp.axis('off')

pp.subplot(1, 4, 3)
pp.imshow(I1up, cmap='gray')
pp.title('Utezeno povprecje')
pp.axis('off')

pp.subplot(1, 4, 4)
pp.imshow(I1g, cmap='gray')
pp.title('Gaussovo jedro')
pp.axis('off')

pp.show()

#%% Naloga 2
# V modulu funkcije ustvarimo funkcijo gaussianKernel2d.

# Naloga a)
Kg1 = funkcije.gaussianKernel2d(1)
Kg2 = funkcije.gaussianKernel2d(2)
Kg4 = funkcije.gaussianKernel2d(4)
I1Kg1 = funkcije.conv2d(I1, Kg1, boundary='constant')
I1Kg2 = funkcije.conv2d(I1, Kg2, boundary='constant')
I1Kg4 = funkcije.conv2d(I1, Kg4, boundary='constant')

def surf(gk, subplot, color='r', rstride=1, cstride=1, shade=True):
  ax = pp.gcf().add_subplot(subplot, projection='3d')
  x, y = np.arange(gk.shape[0]), np.arange(gk.shape[1])
  x = x
  y = y
  Yk, Xk = np.meshgrid(y, x, indexing='ij')
  ax.plot_surface(
    Xk, Yk, gk, 
    color=color, rstride=rstride, cstride=cstride, shade=shade)

# Gaussova konvolucijska jedra za vrednosti sigma 1, 2, in 4.
pp.figure()
pp.suptitle('Gaussova jedra za vrednosti sigma 1, 2 in 4')

pp.subplot(2, 3, 1)
pp.imshow(Kg1, cmap='gray')
pp.title('sigma = 1, velikost={}'.format(Kg1.shape))

pp.subplot(2, 3, 2)
pp.imshow(Kg2, cmap='gray')
pp.title('sigma = 2, velikost={}'.format(Kg2.shape))

pp.subplot(2, 3, 3)
pp.imshow(Kg4, cmap='gray')
pp.title('sigma = 4, velikost={}'.format(Kg4.shape))
surf(Kg1, 234)
surf(Kg2, 235)
surf(Kg4, 236)

pp.figure()
pp.suptitle('Nezveznosti na robu slike po konvoluciji z Gaussovim jedrom.')

pp.subplot(1, 3, 1)
pp.imshow(I1, cmap='gray')
pp.title('Izvirna slika')

pp.subplot(1, 3, 2)
pp.imshow(I1Kg1, cmap='gray')
pp.title('Filtrirana s sigma=1')

pp.subplot(1, 3, 3)
pp.imshow(I1Kg2, cmap='gray')
pp.title('Filtrirana s sigma=2')

pp.show()

# Naloga b)
# S tem, ko postavimo vrednosti sivinskih elementov izven definicijskega 
# območja slike na 0, vnašamo nezveznosti v zunanji rob slike, in sicer
# v širini polovice konvolucijskega jedra (slika \ref{fig:robSlike}).
# Izboljšanje bi lahko dosegli z uporabo sivinske vrednosti najbližjega
# slikovnega elementa iz definicijskega območja slike namesto vrednosti 0.

# Naloga c)
# V modulu funkcije ustvarimo funkcijo imPad2d in prilagodimo funkcijo conv2d.

# Naloga d)
# Razširjeni del slike mora znašati polovico velikosti konvolucijskega jedra.
# floor((n - 1)/2)

#%% Naloga 3 
# V modulu funkcije ustvarimo funkcijo conv1d.

#%% Naloga 4 
# V modulu funkcije ustvarimo funkcijo gaussianKernel1d.

# Glavna prednost filtriranja slik z dvema zaporednima 1D konvolucijama pred
# enim filtriranjem z 2D konvolucijo je v numerični zahtevnosti.
# V prvem primeru zahtevnost raste linearno z velikostjo konvolucijskega
# jedra, v drugem primeru pa s kvadratom velikosti konvolucijskega jedra.

#%% Naloga 5
# V modulu funkcije ustvarimo funkcijo imGaussFilt2D.

I1f = I1.astype(np.float)
I1C2d = funkcije.conv2d(I1f, Kg1, boundary='constant')
I1C1d = np.zeros(I1.shape)
k = funkcije.gaussianKernel1d(1.0)
for i in range(I1.shape[0]):
    I1C1d[i, :] = funkcije.conv1d(I1f[i, :], k, boundary='constant')
for i in range(I1.shape[1]):
    I1C1d[:, i] = funkcije.conv1d(I1C1d[:, i], k, boundary='constant')

pp.figure()
pp.suptitle('Primerjava 2D in 2 x 1D konvolucije')

pp.subplot(1, 3, 1)
pp.imshow(I1C1d, cmap='gray')
pp.title('2 x 1D konvolucija')
pp.axis('off')

pp.subplot(1, 3, 2)
pp.imshow(I1C2d, cmap='gray')
pp.title('2D konvolucija')
pp.axis('off')

pp.subplot(1, 3, 3)
pp.imshow(I1C2d - I1C1d, cmap='gray')
pp.title('Razlika slik')
pp.axis('off')

pp.show()

#%% Naloga 6
# V modulu funkcije ustvarimo funkcijo imSharpen2d.

# Naloga a)
I1sm = funkcije.imSharpen2d(I1, 'mask', c=1.0, sigma=1.0)
I1sm = np.clip(I1sm, 0, 255)
I1sl = funkcije.imSharpen2d(I1, 'laplace', c=1.0)
I1sl = np.clip(I1sl, 0, 255) 

pp.figure()
pp.suptitle('Primerjava ostrenja z masko in Laplaceovim operatorjem')

pp.subplot(1, 3, 1)
pp.imshow(I1, cmap='gray')
pp.title('Izvirna slika')
pp.axis('off')

pp.subplot(1, 3, 2)
pp.imshow(I1sm, cmap='gray')
pp.title('Ostrenje z masko')
pp.axis('off')

pp.subplot(1, 3, 3)
pp.imshow(I1sl, cmap='gray')
pp.title('Ostrenje z Laplaceom')
pp.axis('off')

pp.show()

# Naloga b)
pp.figure()
pp.suptitle('Ostrenje z masko')
pp.subplot(1, 4, 1)
pp.imshow(I1, cmap='gray')
for index, c in enumerate((0.5, 1, 2.0)):
  img = funkcije.imSharpen2d(I1, 'mask', c=c, sigma=1.0)
  img = np.clip(img, 0, 255)
  pp.subplot(1, 4, index + 2)
  pp.imshow(img, cmap='gray')
  pp.title('c = {:.1f}'.format(c))

pp.figure()
pp.suptitle('Ostrenje z Laplaceom')
pp.subplot(1, 4, 1)
pp.imshow(I1, cmap='gray')
for index, c in enumerate((0.5, 1, 2.0)):
  img = funkcije.imSharpen2d(I1, 'laplace', c=c)
  img = np.clip(img, 0, 255)
  pp.subplot(1, 4, index + 2)
  pp.imshow(img, cmap='gray')
  pp.title('c = {:.1f}'.format(c))

#%% Naloga 7
# V modulu funkcije ustvarimo funkcijo imStatFilt2d.

I2m = funkcije.imStatFilt2d(I2, 3, 'median')
I2g = funkcije.imGaussFilt2d(I2, 0.5)

pp.figure()
pp.suptitle('Primerjava medianinega in Gaussovega filtra.')

pp.subplot(1, 3, 1)
pp.imshow(I2, cmap='gray')
pp.title('Izvirna slika')
pp.axis('off')

pp.subplot(1, 3, 2)
pp.imshow(I2m, cmap='gray')
pp.title('Medianin filter')
pp.axis('off')

pp.subplot(1, 3, 3)
pp.imshow(I2g, cmap='gray')
pp.title('Gaussov filter')
pp.axis('off')

pp.show()

#%% Naloga 8
Ka = np.array([[1, 1, 1],
               [1, 0, 1],
               [1, 1, 1]], dtype=np.float)/8.0

I2a = funkcije.conv2d(I2, Ka, boundary='reflect')

pp.figure()

pp.subplot(1, 3, 1)
pp.title('Izvirna slika')
pp.imshow(I2, cmap='gray')

pp.subplot(1, 3, 2)
pp.title('Mediana 3 x 3')
pp.imshow(I2m, cmap='gray')

pp.subplot(1, 3, 3)
pp.title('Povprečje sosedov')
pp.imshow(I2a,cmap='gray')

pp.show()

# Filtriranje z mediano bo uspešno odstranilo šum tipa sol in poper tudi
# v primeru, ko je več sosednjih slikovnih elementov podvrženo šumu
# (največ polovica sosednjih slikovnih elementov).
# Filtriranje s povprečjem sosednjih slikovnih elementov bo uspešno le,
# če sosednji slikovni elementi niso podvrženi šumu tipa sol in poper.

#%% Naloga 9
# V modulu funkcije ustvarimo funkcijo imGaussFilt3d.

I3f = funkcije.imGaussFilt3d(I3, 0.5)

pp.figure()
pp.suptitle('Filtriranje 3D slike, x=90 in z=90')

pp.subplot(2, 2, 1)
pp.imshow(I3[:,:,90], cmap='gray')
pp.title('Izvirna slika x=90')
pp.axis('off')

pp.subplot(2, 2, 2)
pp.imshow(I3f[:,:,90], cmap='gray')
pp.title('Filtrirana slika x=90')
pp.axis('off')

pp.subplot(2, 2, 3)
pp.imshow(I3[90,:,:].squeeze(), cmap='gray')
pp.title('Izvirna slika z=90')
pp.axis('off')
pp.subplot(2, 2, 4)
pp.imshow(I3f[90,:,:].squeeze(), cmap='gray')
pp.title('Filtrirana slika z=90')
pp.axis('off')

pp.show()

#%% Naloga 10
from scipy.ndimage import convolve, gaussian_filter, median_filter, laplace

# konvolucija z gaussovim jedrom
oimg = convolve(I1, Kg1)

# gaussov filter sigma=1.0, 2D slika
oimg = gaussian_filter(I1, sigma=1.0)

# gaussov filter sigma=1.0, 3D slika
oimg = gaussian_filter(I3, sigma=1.0)

# medianin filter velikosti 3 x 3
oimg = median_filter(I1, [3, 3])

# laplace
oimg = laplace(I1)
