# -*- coding: utf-8 -*-
#%% Poglavje 8

import os.path
import funkcije
import numpy as np
import interp
from matplotlib import pyplot as pp
from PIL import Image as im

# naložimo sliko kalibracijskega objekta
I1 = np.array(im.open('./poglavje_8/cal_projective.png'))
H1, W1 = I1.shape
xg1, yg1 = np.arange(W1), np.arange(H1)
# definiramo referenčne točke
xr1 = np.array([0, 4, 8, 8, 8, 4, 0, 0], dtype=np.float)
yr1 = np.array([0, 0, 0, 3, 6, 6, 6, 3], dtype=np.float)
# definiramo koordinatno mrežo točk kalibrirane slike
x1c = np.arange(0, 8.0 + 0.01, 0.02)
y1c = np.arange(0, 6.0 + 0.01, 0.02)
Y1c, X1c = np.meshgrid(y1c, x1c, indexing='ij')

# naložimo sliko kalibracijske tarče
I2 = np.array(im.open('./poglavje_8/cal_barrel.png'))
H2, W2 = I2.shape
xg2, yg2 = np.arange(W2), np.arange(H2)
# definiramo referenčne točke
xr2 = xr1
yr2 = yr1
# definiramo koordinatno mrežo točk kalibrirane slike
x2c, y2c = x1c, y1c
X2c, Y2c = X1c, Y1c

#%% Naloga 1
# V modulu funkcije ustvarimo funkcijo transformEstimate2D.

#%% Naloga 2

# Naloga a)
# ročno označevanje referenčnih točk
pp.figure()
if os.path.isfile('./poglavje_8/rezultati/ref_pts_1.npy'):
  t = np.load('./poglavje_8/rezultati/ref_pts_1.npy')
else:
  pp.imshow(I1, cmap='gray')
  pp.title(
    'Označi oglišča in razpolovišča stranic. '
    'Prični levo zgoraj, nadaljuj v smeri urinega kazalca.')
  t = np.asarray(pp.ginput(8, timeout=600))
  np.save('./poglavje_8/rezultati/ref_pts_1.npy', t)
pp.close()
u1, v1 = t[:,0], t[:,1]

# kalibracija z interpolacijsko afino preslikavo
# uporabimo levo zgornje, desno zgornje in desno spodnje krajišče
inds = [0, 2, 4]
T1Ai, err1Ai = funkcije.transformEstimate2d(
  xr1[inds], yr1[inds], u1[inds], v1[inds], 'affine')
x1Ai, y1Ai = funkcije.ptTransform2d(T1Ai, u1, v1, inverse=True)
tmpx, tmpy = funkcije.ptTransform2d(T1Ai, X1c, Y1c)
I1Ai = interp.interp2(tmpx, tmpy, xg1, yg1, I1, cval=255)

# kalibracija z aproksimacijsko afino preslikavo
T1A, err1A = funkcije.transformEstimate2d(
  xr1, yr1, u1, v1, 'affine')
x1A, y1A = funkcije.ptTransform2d(T1A, u1, v1, inverse=True)
tmpx, tmpy = funkcije.ptTransform2d(T1A, X1c, Y1c)
I1A = interp.interp2(tmpx, tmpy, xg1, yg1, I1, cval=255)

# kalibracija s projektivno preslikavo
T1P, err1P = funkcije.transformEstimate2d(
  xr1, yr1, u1, v1, 'projective', animate=False)
x1P, y1P = funkcije.ptTransform2d(T1P, u1, v1, inverse=True)
tmpx, tmpy = funkcije.ptTransform2d(T1P, X1c, Y1c)
I1P = interp.interp2(tmpx, tmpy, xg1, yg1, I1, cval=255)

print('Kalibracija slike cal_projective.png:')
print('  Napaka za interp. afini model:', err1Ai)
print('  Napaka za aproks. afini model:', err1A)
print('  Napaka za proj. model:', err1P)

pp.figure()

pp.subplot(2, 2, 1)
pp.plot(xr1, yr1, 'xr', label='Izvirna')
pp.plot(x1A, y1A, 'xb', label='Afina')
pp.plot(x1P, y1P, 'xg', label='Projektivna')
pp.legend(loc='upper right')

pp.subplot(2, 2, 2)
pp.imshow(I1, cmap='gray')
pp.title('Izvirna')

pp.subplot(2, 2, 3)
pp.imshow(I1A, cmap='gray')
pp.title('Afina')

pp.subplot(2, 2, 4)
pp.imshow(I1P, cmap='gray')
pp.title('Projektivna')

pp.show()

# Naloga b)
# Za kalibracijo zajete slike je potrebno uporabiti projektivno preslikavo.
# Koren povprečne kvadratne evklidske razdalje med korespondenčnimi točkami v
# prostoru zajete slike znaša približno 50 slikovnih elementov za
# aproksimacijsko afino in 0,5 slikovnega elementa za projektivno preslikavo.

#%% Naloga 3

# Naloga a)
# ročno označevanje referenčnih točk
pp.figure()
if os.path.isfile('./poglavje_8/rezultati/ref_pts_2.npy'):
  t = np.load('./poglavje_8/rezultati/ref_pts_2.npy')
else:
  pp.imshow(I2, cmap='gray')
  pp.title(
    'Izberi oglišča in razpolovišča stranic. '
    'Prični levo zgoraj.')
  t = np.asarray(pp.ginput(8, timeout=600))
  np.save('./poglavje_8/rezultati/ref_pts_2.npy', t)
pp.close()
u2, v2 = t[:,0], t[:,1]

# kalibracija z afino aproksimacijsko preslikavo
T2A, err2A = funkcije.transformEstimate2d(
  xr2, yr2, u2, v2, 'affine')
tmpx, tmpy = funkcije.ptTransform2d(T2A, X2c, Y2c)
I2A = interp.interp2(tmpx, tmpy, xg2, yg2, I2, cval=255)

# kalibracija s projektivno preslikavo
T2P, err2P = funkcije.transformEstimate2d(
  xr2, yr2, u2, v2, 'projective')
tmpx, tmpy = funkcije.ptTransform2d(T2P, X2c, Y2c)
I2P = interp.interp2(tmpx, tmpy, xg2, yg2, I2, cval=255)

# a - kalibracija z radialnim Brownovim modelom
T2RP, T2RB, err2RP = funkcije.transformEstimate2d(
  xr2, yr2, u2, v2, 'radial', animate=True)
tmpx, tmpy = funkcije.ptTransformRadial2d(T2RB, X2c, Y2c)
tmpx, tmpy = funkcije.ptTransform2d(T2RP, tmpx, tmpy)
I2RP = interp.interp2(tmpx, tmpy, xg2, yg2, I2, cval=255)

print('Kalibracija slike cal_barrel.png:')
print('  Napaka za afini model preslikave:', err2A)
print('  Napaka za proj. model preslikave:', err2P)
print('  Napaka za proj. in rad. model preslikave:', err2RP)

pp.figure()

pp.subplot(2, 2, 1)
pp.imshow(I2, cmap='gray')
pp.title('Zajeta')

pp.subplot(2, 2, 2)
pp.imshow(I2A, cmap='gray')
pp.title('Afina preslikava')

pp.subplot(2, 2, 3)
pp.imshow(I2P, cmap='gray')
pp.title('Projektivna preslikava')

pp.subplot(2, 2, 4)
pp.imshow(I2RP, cmap='gray')
pp.title('Radialna preslikava')

tmpx, tmpy = funkcije.ptTransformRadial2d(T2RB, xr2, yr2)
x2RP, y2RP = funkcije.ptTransform2d(T2RP, tmpx, tmpy)

pp.figure()

pp.plot(u2, v2, 'xr', label='Zajeta')
pp.plot(x2RP, y2RP, 'xb', label='Radialna + Projektivna')
pp.legend(loc='upper right')

pp.show()

# Naloga b)
# Z nekajkratno ponovitvijo postopka kalibracije iz predhodne točke pridemo
# do ocene napake, ki znaša približno 0,3 slikovnega elementa zajete slike.
