# -*- coding: utf-8 -*-
#%% Poglavje 9

import funkcije
import numpy as np
import interp
from matplotlib import pyplot as pp
from PIL import Image as im

# definiramo spremenljivke pomožne funkcije
I1ref = np.asarray(im.open('./poglavje_9/reg_reference.png'))
I1input1 = np.asarray(im.open('./poglavje_9/reg_input1.png'))
I1input2 = np.asarray(im.open('./poglavje_9/reg_input2.png'))
I1input3 = np.asarray(im.open('./poglavje_9/reg_input3.png'))

I2MR1 = np.asarray(im.open('./poglavje_9/reg_MR1.png'))
I2MR2 = np.asarray(im.open('./poglavje_9/reg_MR2.png'))

I3T1 = np.asarray(im.open('./poglavje_9/reg_T1.png'))
I3T2 = np.asarray(im.open('./poglavje_9/reg_T2.png'))
I3SD = np.asarray(im.open('./poglavje_9/reg_SD.png'))

tx = ty = np.arange(-15.0, 15.0 + 1.0)
Ty, Tx = np.meshgrid(ty, tx, indexing='ij')
Txf, Tyf = Tx.flatten(), Ty.flatten()

# izračuna vrednosti vseh mer podobnosti za polje premikov Tx, Ty
def rigidSM(imgA, imgB, tx, ty):
  Ia, Ib = imgA.astype(np.float), imgB.astype(np.float) 
  txshape = tx.shape
  tx = tx.flatten().astype(np.float)
  ty = ty.flatten().astype(np.float)

  MSE = np.zeros_like(tx)
  MAE = np.zeros_like(tx)
  CC = np.zeros_like(tx)
  MI = np.zeros_like(tx)

  for i in range(tx.size):
    txi, tyi = tx[i], ty[i]
    T = np.array([[1, 0, txi],[0, 1, tyi],[0, 0, 1]], dtype=np.float)
    MSE[i] = funkcije.imSM(Ia, 
      funkcije.imTransform2d(Ib, T, expand='same'), 'MSE')
    MAE[i] = funkcije.imSM(Ia, 
      funkcije.imTransform2d(Ib, T, expand='same'), 'MAE')
    CC[i] = funkcije.imSM(Ia,
      funkcije.imTransform2d(Ib, T, expand='same'), 'CC')
    MI[i] = funkcije.imSM(Ia,
      funkcije.imTransform2d(Ib, T, expand='same'), 'MI')
  
  MSE.shape = MAE.shape = CC.shape = MI.shape = txshape
  return MSE, MAE, CC, MI

# izpiše premik pri optimalni vrednosti mer podobnosti
def printRigidOpt(MSE, MAE, CC, MI, tx, ty):
  if MSE is not None:
    ind = np.argmin(MSE)
    print('Minimum MSE={} pri tx={}, ty={}.'.format(
      MSE.flatten()[ind], tx.flatten()[ind], ty.flatten()[ind]))
  if MAE is not None:
    ind = np.argmin(MAE)
    print('Minimum MAE={} pri tx={}, ty={}.'.format(
      MAE.flatten()[ind], tx.flatten()[ind], ty.flatten()[ind]))
  if CC is not None:
    ind = np.argmax(CC)
    print('Maximum CC={} pri tx={}, ty={}.'.format(
      CC.flatten()[ind], tx.flatten()[ind], ty.flatten()[ind]))
  if MI is not None:
    ind = np.argmax(MI)
    print('Maximum MI={} pri tx={}, ty={}.'.format(
      MI.flatten()[ind], tx.flatten()[ind], ty.flatten()[ind]))

# izriše kriterijsko funkcijo kot sliko
def showSMs(MSE, MAE, CC, MI, tx, ty, title=''):
  # extent = [left, right, bottom, top]
  extent = [tx.min(), tx.max(), ty.max(), ty.min()]

  pp.figure()

  pp.suptitle(title)
  pp.subplot(1, 4, 1)
  pp.imshow(MSE, extent=extent), 
  pp.title('MSE'), pp.colorbar()

  pp.subplot(1, 4, 2)
  pp.imshow(MAE, extent=extent)
  pp.title('MAE'), pp.colorbar()

  pp.subplot(1, 4, 3)
  pp.imshow(CC, extent=extent)
  pp.title('CC'), pp.colorbar()

  pp.subplot(1, 4, 4)
  pp.imshow(MI, extent=extent)
  pp.title('MI'), pp.colorbar()

  pp.show()
#%% Naloga 1

# Naloga a)
# V modulu funkcije ustvarimo funkcijo imSM.

# Naloga b)
MSEref, MAEref, CCref, MIref = rigidSM(I1ref, I1ref, Tx, Ty)
showSMs(MSEref, MAEref, CCref, MIref, tx, ty,
  'Mere podobnosti med premaknjeno in izvirno sliko reg_reference')

# Naloga c)
# Meri podobnosti MAE in MSE izkazujeta minimum v okolici optimalne vrednosti.
# Mera CC izkazuje vrednost 1 v okolici optimalne vrednosti.
#
ind = np.argmin(MSEref)
print(
  'Mera podaobnosti kot funkcija togega premika '
  'za sliko reg_reference:\n')
printRigidOpt(MSEref, MAEref, CCref, MIref, Tx, Ty)

# Naloga d)
MSEi1, MAEi1, CCi1, MIi1 = rigidSM(I1ref, I1input1, Tx, Ty)
print('\nPoravnava slike reg_reference in reg_input1:')
printRigidOpt(MSEi1, MAEi1, CCi1, MIi1, Tx, Ty)

MSEi2, MAEi2, CCi2, MIi2 = rigidSM(I1ref, I1input2, Tx, Ty)
print('\nPoravnava slike reg_reference in reg_input2:')
printRigidOpt(MSEi2, MAEi2, CCi2, MIi2, Tx, Ty)

MSEi3, MAEi3, CCi3, MIi3 = rigidSM(I1ref, I1input3, Tx, Ty)
print('\nPoravnava slike reg_reference in reg_input3:')
printRigidOpt(MSEi3, MAEi3, CCi3, MIi3, Tx, Ty)

# Naloga e)
# Medsebojna informacija je statistična mera podobnosti, katere vrednost
# izhaja iz verjetnostnih porazdelitev sivinskih vrednosti slik.

#%% Naloga 2

# Naloga a)
# V modulu funkcije ustvarimo funkcijo imRigidRegister.
X2CC = []
T2CC = []
f2CC = []
N2CC = []
for sm in ['MSE', 'MAE', 'CC', 'MI']:
  X2CCi, T2CCi, f2CCi, N2CCi = funkcije.imRigidRegister(
    I2MR1, I2MR2, sm, [5, 5, 0.1], animate=True)
  print(
    '\nParametri toge poravnave slik reg_MR1 in reg_MR2 s '
    'SM=''{}'': tx={}, ty={}, fi={} rad, f={}, N={}'.format(
    sm, X2CCi[0], X2CCi[1], X2CCi[2], f2CCi, N2CCi))
  X2CC.append(X2CCi)
  T2CC.append(T2CCi)
  f2CC.append(f2CCi)
  N2CC.append(N2CCi)

# Naloga b)
# Postopek poravnave se vedno ne konča, tako da sta sliki poravnani.
# Rezultat je odvisen od izbrane mere podobnosti in
# začetnega približka preslikave.

#%% Naloga 3

# Nalogi a) in b)
tx3 = np.arange(-15.0, 15.0 + 1.0)
ty3 = np.arange(-30.0, 0 + 1.0)
Ty3, Tx3 = np.meshgrid(ty3, tx3, indexing='ij')
 
MSET1, MAET1, CCT1, MIT1 = rigidSM(I3SD, I3T1, Tx3, Ty3)
print('\nMere podobnosti med slikama reg_SD in reg_T1:')
printRigidOpt(MSET1, MAET1, CCT1, MIT1, Tx3, Ty3)
showSMs(MSET1, MAET1, CCT1, MIT1, tx3, ty3,
        'Poravnava reg_SD in reg_T1')

MSET2, MAET2, CCT2, MIT2 = rigidSM(I3SD, I3T2, Tx3, Ty3)
print('\nMere podobnosti med slikama reg_SD in reg_T2:')
printRigidOpt(MSET2, MAET2, CCT2, MIT2, Tx3, Ty3)
showSMs(MSET2, MAET2, CCT2, MIT2, tx3, ty3,
        'Poravnava reg_SD in reg_T2')

# Naloga c)
# Izberemo medsebojno informacijo kot mero podobnosti
ind = np.argmax(MIT1)
x0 = [Tx3.flatten()[ind], Ty3.flatten()[ind], 0]
X3CCT1, T3CCT1, f3CCT1, N3CCT1 = funkcije.imRigidRegister(
  I3SD, I3T1, 'MI', x0=x0, animate=True)
print(
  'Parametri toge preslikave, ki poravna sliki reg_SD in reg_T1: '
  'tx={}, ty={}, fi={} rad, f={}, N={}'.format(
  X3CCT1[0], X3CCT1[1], X3CCT1[2], f3CCT1, N3CCT1))

ind = np.argmax(MIT2)
x0 = [Tx3.flatten()[ind], Ty3.flatten()[ind], 0]
X3CCT2, T3CCT2, f3CCT2, N3CCT2 = funkcije.imRigidRegister(
   I3SD, I3T2, 'MI', x0=x0, animate=True)
print(
  'Parametri toge preslikave, ki poravna sliki reg_SD in reg_T2: '
  'tx={}, ty={}, fi={} rad, f={}, N={}'.format(
  X3CCT2[0], X3CCT2[1], X3CCT2[2], f3CCT2, N3CCT2))

pp.show()
